@@ -25,7 +25,7 @@ use Kossy::Request;
 use Kossy::Response;
 use HTTP::Headers::Fast;
 
-our $VERSION = '0.61';
+our $VERSION = '0.62';
 our @EXPORT = qw/new root_dir psgi build_app _router _connect get post router filter _wrap_filter/;
 
 our $XSLATE_CACHE = 1;
@@ -513,6 +513,35 @@ validate parameters using L<Kossy::Validator>
 
 These methods are the accessor to raw values. 'raw' means the value is not decoded.
 
+=item body_parameters
+
+Accessor to decoded body parameters. It's Hash::MultiValue object.
+
+=item query_parameters
+
+Accessor to decoded query parameters. It's Hash::MultiValue object.
+
+=item json_parameters
+
+Accessor to decoded JSON body parameters. It's B<NOT> Hash::MultiValue object.
+
+    post '/api' => sub {
+        my ($self, $c) = @_;
+        my $foo = $c->req->json_parameters->{foo}; # bar
+    };
+
+    # requrest
+    # $ua->requrest(
+    #     HTTP::Request->new(
+    #         "POST",
+    #         "http://example.com/api",
+    #         [ "Content-Type" => 'application/json', "Content-Length" => 13 ],
+    #         '{"foo":"bar"}'
+    #     )
+    # );
+
+NOTE: Not need to set C<kossy.request.parse_json_body> to 1
+
 =back
 
 =head1 Kossy::Response
